import "Turbine";
import "Turbine.Gameplay";
import "MinstrelBuffs.MinstrelBuffWindow.BuffWindow";
import "MinstrelBuffs.MinstrelBuffWindow.MinstrelEffects";
import "MinstrelBuffs.MinstrelBuffWindow.MinstrelBuffCommand";
import "MinstrelBuffs.MinstrelBuffWindow.Settings";
import "MinstrelBuffs.MinstrelBuffWindow.EventHelper";
import "MinstrelBuffs.MinstrelBuffWindow.OptionWindow";

MinstrelBuffMain = class();

function MinstrelBuffMain:Constructor()
	self.settings = MinstrelBuffs.MinstrelBuffWindow.Settings();
	self.BuffWindow = MinstrelBuffs.MinstrelBuffWindow.BuffWindow(self.settings);
	self.optionWindow = OptionWindow(self.settings);

	-- Options support for Turbine PluginManager
	plugin.GetOptionsPanel = function() return self:GetOptionControl(); end
end

function MinstrelBuffMain:WriteBuffsToChat()
	local placeCount = self:GetBuffCount();
	local effectCount = EffectList:GetCount();
	Turbine.Shell.WriteLine( "Current buffs: " .. placeCount );
	for	i = 1, effectCount, 1 do
		local effect = EffectList:Get(i);
		Turbine.Shell.WriteLine( effect:GetName() );		
	end
end

function MinstrelBuffMain:ShowWindow()
	self.EffectChangedCallback = function(sender, args) self:RefreshEffectDisplay(); end
	self.InCombatChanged = function(sender, args) self:RefreshVisibility(); end
	AddCallback(EffectList, "EffectAdded", self.EffectChangedCallback);
	AddCallback(EffectList, "EffectRemoved", self.EffectChangedCallback);
	AddCallback(EffectList, "EffectCleared", self.EffectChangedCallback);
	AddCallback(LocalPlayer, "InCombatChanged", self.InCombatChanged);

	self.PositionChangedCallback = function(sender, args) self:SaveMainPositionSettings(); end
	AddCallback(self.BuffWindow, "PositionChanged", self.PositionChangedCallback);

	self.BuffWindow:Start();
	self:RefreshEffectDisplay();
end

function MinstrelBuffMain:HideWindow()
	if (self.EffectChangedCallback ~= nil) then
		RemoveCallback(EffectList, "EffectAdded", self.EffectChangedCallback);
		RemoveCallback(EffectList, "EffectRemoved", self.EffectChangedCallback);
		RemoveCallback(EffectList, "EffectCleared", self.EffectChangedCallback);
		RemoveCallback(LocalPlayer, "InCombatChanged", self.InCombatChanged);
		self.EffectChangedCallback = nil;
	end	
	
	if (self.PositionChangedCallback ~= nil) then
		RemoveCallback(self.BuffWindow, "PositionChanged", self.PositionChangedCallback);
		self.PositionChangedCallback = nil;
	end
	self.BuffWindow:Stop();
end

function MinstrelBuffMain:UiLocked(locked)
	self:SaveMainPositionSettings();
	self.BuffWindow:UiLocked(locked);
end

function MinstrelBuffMain:ShowOptions()
	--self.optionWindow = OptionWindow(self.settings);
	self.optionWindow:Show();
end

function MinstrelBuffMain:GetOptionControl()
	--self.optionWindow = OptionWindow(self.settings);
	return self.optionWindow:GetOptionControl();
end

function MinstrelBuffMain:SaveMainPositionSettings()
	--Turbine.Shell.WriteLine("SavePosition");
	if (self.BuffWindow ~= nil) then
		local posx, posy = self.BuffWindow:GetPosition();
		self.settings:SetMainPos(posx, posy);		
	end
end

function MinstrelBuffMain:GetLocalization()
	return self.settings:GetEffectLocalizationNames();
end

function MinstrelBuffMain:RefreshEffectDisplay()
	local effectCount = EffectList:GetCount();
	local balladEffectPlace = 1;
	local anthemEffectPlace = 1;
	local sortedEffects = {};
	local isAnthemFixed = self.settings:GetFixedAnthemPositions();
	local showAnthems = self.settings:GetShowAnthems();
	for	i = 1, effectCount, 1 do
		sortedEffects[i] = EffectList:Get(i);
	end
	table.sort(sortedEffects, function(a,b) return a:GetStartTime() < b:GetStartTime() end);
	
	if (isAnthemFixed == true) then
		for i = 1, 6, 1 do
			self.BuffWindow:HideAnthem(i);
		end
	end

	for	i = 1, effectCount, 1 do
		--local effect = EffectList:Get(i);
		local effect = sortedEffects[i];
		if (balladEffectPlace <= 3 and self:IsValidBalladEffect(effect:GetName())) then
			self.BuffWindow:ShowEffect(effect, balladEffectPlace);
			balladEffectPlace = balladEffectPlace + 1;
		end
		if (showAnthems == true) then
			if (isAnthemFixed == false) then
				if (anthemEffectPlace <= 6 and self:IsValidHymnEffect(effect:GetName())) then
					self.BuffWindow:ShowAnthem(effect, anthemEffectPlace);
					anthemEffectPlace = anthemEffectPlace + 1;
				end
			else
				if (self:IsValidHymnEffect(effect:GetName())) then
					local anthemEffectPosition = self:GetHymnEffectPosition(effect:GetName());
					self.BuffWindow:ShowAnthem(effect, anthemEffectPosition);
				end
			end
		end
	end
	
	if (balladEffectPlace <= 3) then
		for i = balladEffectPlace, 3, 1 do
			self.BuffWindow:HideEffect(i);
		end
	end
	if (anthemEffectPlace <= 6 and isAnthemFixed == false) then
		for i = anthemEffectPlace, 6, 1 do
			self.BuffWindow:HideAnthem(i);
		end
	end
end

function MinstrelBuffMain:IsValidBalladEffect(effectName)
	if (self:GetLocalization() == "fr") then
		return ValidBalladEffectsFr[effectName] ~= nil;
	elseif (self:GetLocalization() == "de") then
		return ValidBalladEffectsDe[effectName] ~= nil;
	else
		return ValidBalladEffectsEn[effectName] ~= nil;
	end
end

function MinstrelBuffMain:IsValidHymnEffect(effectName)
	if (self:GetLocalization() == "fr") then
		return ValidHymnEffectsFr[effectName] ~= nil;
	elseif (self:GetLocalization() == "de") then
		return ValidHymnEffectsDe[effectName] ~= nil;
	else
		return ValidHymnEffectsEn[effectName] ~= nil;
	end
end

function MinstrelBuffMain:GetHymnEffectPosition(effectName)
	if (self:GetLocalization() == "fr") then
		return ValidHymnEffectsFr[effectName];
	elseif (self:GetLocalization() == "de") then
		return ValidHymnEffectsDe[effectName];
	else
		return ValidHymnEffectsEn[effectName];
	end
end

function MinstrelBuffMain:GetBuffCount()
	local count = 0;
	count = EffectList:GetCount();
	return count;
end

function MinstrelBuffMain:RefreshVisibility()
	self.BuffWindow:SetVisibility();
end

mbMain = MinstrelBuffMain();
--mbCommand = MinstrelBuffs.MinstrelBuffWindow.MinstrelBuffCommand(mbMain);
MinstrelBuffCommand.MinstrelBuffMain = mbMain;
Turbine.Shell.AddCommand( "minstrelbuff", MinstrelBuffCommand);
Turbine.Shell.AddCommand( "mb", MinstrelBuffCommand);

LocalPlayer = Turbine.Gameplay.LocalPlayer.GetInstance();
EffectList = LocalPlayer:GetEffects();
mbMain:ShowWindow();
MinstrelBuffCommand:ShowHelpText();