import "Turbine.UI.Lotro"

OptionWindow = class(Turbine.UI.Lotro.Window);

function OptionWindow:Constructor(settings)
	Turbine.UI.Lotro.Window.Constructor(self);

	self.settings = settings;
	self:SetPosition( 200, 200 );
	self:SetText("MinstrelBuff - Options");
	self:SetSize( 270, 400 );
	self:SetVisible(false);
	
	self:PlaceControls(self);
end

function OptionWindow:GetOptionControl()
	local control = Turbine.UI.Control();
	control:SetSize( 270, 400 );
	self:PlaceControls(control);
	return control;
end

function OptionWindow:PlaceControls(parent)
	-- Combat Visibility
	self.combatVisibleCheckBox = Turbine.UI.Lotro.CheckBox();
	self.combatVisibleCheckBox:SetParent(parent);
	self.combatVisibleCheckBox:SetText("Only visible in combat");
	self.combatVisibleCheckBox:SetPosition( 20, 40 );
	self.combatVisibleCheckBox:SetSize( 230, 25 );
	self.combatVisibleCheckBox:SetChecked(self.settings:GetEffectWindowOnlyVisibleInCombat());
	self.combatVisibleCheckBox.CheckedChanged = function (sender, args) self:EffectWindowOnlyVisibleInCombatChanged(); end

	-- Theme options
	local themeLabel = Turbine.UI.Label();
	themeLabel:SetPosition( 20, 70 );
	themeLabel:SetSize( 230, 20 );
	themeLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	themeLabel:SetText("Select Theme:");
	themeLabel:SetParent(parent);
	
	local themeList = self.settings:GetThemeList();
	local themeIndex = self.settings:GetThemeIndex();
	local themeCount = table.getn(themeList);
	
	self.listBox = Turbine.UI.ListBox();
	self.listBox:SetParent( parent );
	self.listBox:SetBackColor( Turbine.UI.Color( 0.2, 0.2, 0.2 ) );
	self.listBox:SetPosition( 20, 100 );
	self.listBox:SetSize( 230, 20 * themeCount );

	for i = 1, themeCount do
		local listItem = Turbine.UI.Label();
		listItem:SetSize( 230, 20 );
		listItem:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
		listItem:SetText(themeList[i].name);
		self.listBox:AddItem( listItem );
	end
	self.listBox:SetSelectedIndex(themeIndex);
	self:ColorListBoxIndex(self.listBox);
	self.listBox.SelectedIndexChanged = function (sender, args) self:ThemeChanged(sender); end
end

function OptionWindow:Show()
	self:SetVisible(true);
end

function OptionWindow:ThemeChanged(sender)
	self:ColorListBoxIndex(sender);
	self.settings:SetThemeIndex(sender:GetSelectedIndex());	
end

function OptionWindow:ColorListBoxIndex(sender)
	for i = 1, sender:GetItemCount() do
		local listItem = sender:GetItem(i);
		if (i == sender:GetSelectedIndex()) then
			listItem:SetBackColor( Turbine.UI.Color( 1, 0.4, 0.4, 0.4 ) );
		else
			listItem:SetBackColor( Turbine.UI.Color( 1, 0.3, 0.3, 0.3 ) );
		end
	end
end

function OptionWindow:EffectWindowOnlyVisibleInCombatChanged()
	self.settings:SetEffectWindowOnlyVisibleInCombat(self.combatVisibleCheckBox:IsChecked());
end